@@ -410,10 +410,15 @@ static int
 write_certificate(X509 *cert, const char serial[], const char dir[]) {
     BIO *bp=NULL;
     char *path;
-    int rval = -1;
+    int rval = -1, fd;
 
     path = malloc(strlen(dir)+strlen(serial)+strlen("/.pem")+1);
     sprintf(path, "%s/%s.pem", dir, serial);
+    if ((fd = open(path, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)) < 0) {
+        myproxy_log("failed to create %s: %s", path, strerror(errno));
+        goto error;
+    }
+    close(fd);
 	if ((bp=BIO_new(BIO_s_file())) == NULL) {
         myproxy_debug("BIO_new(BIO_s_file()) failed");
         goto error;
@@ -542,6 +547,9 @@ generate_certificate( X509_REQ                 *request,
   }
   fclose(issuer_cert_file);
 
+  myproxy_debug("certificate_issuer_cert: %s",
+                server_context->certificate_issuer_cert );
+
   X509_set_issuer_name(cert, X509_get_subject_name(issuer_cert));
 
   X509V3_set_ctx(ctxp, issuer_cert, cert, NULL, NULL, 0);
@@ -715,7 +723,8 @@ generate_certificate( X509_REQ                 *request,
     ssl_error_to_verror();
     goto error;
   } else {
-    myproxy_debug("CAkey: %s", server_context->certificate_issuer_key );
+    myproxy_debug("certificate_issuer_key: %s",
+                  server_context->certificate_issuer_key );
   }
 
   if (!X509_check_private_key(issuer_cert,cakey)) {
@@ -1227,6 +1236,13 @@ handle_certificate(unsigned char            *input_buffer,
 
 }
 
+int is_certificate_authority_configured(myproxy_server_context_t
+                                        *context) {
+    return (context->certificate_issuer_program ||
+            context->certificate_issuer_cert);
+}
+
+
 void get_certificate_authority(myproxy_socket_attrs_t   *server_attrs, 
 			       myproxy_creds_t          *creds,
 			       myproxy_request_t        *client_request,
