@@ -1,5 +1,4 @@
 #include "myproxy_common.h"	/* all needed headers included here */
-#include "pubcookie.h"
 
 #if defined(HAVE_LIBPAM)
 #include "auth_pam.h"
@@ -39,6 +38,7 @@ auth_passwd_get_status(struct myproxy_creds *creds, char *client_name,
     
     if (myproxy_creds_exist(creds->username, creds->credname) == 1 &&
 	myproxy_creds_encrypted(creds) == 1) {
+	verror_clear();
 	return AUTHORIZEMETHOD_REQUIRED;
     }
 
@@ -53,10 +53,6 @@ auth_passwd_get_status(struct myproxy_creds *creds, char *client_name,
     }
 #endif
 
-   if (config->pubcookie_cert) {
-       return AUTHORIZEMETHOD_SUFFICIENT;
-   }
-
     return AUTHORIZEMETHOD_DISABLED;
 }
 
@@ -82,290 +78,6 @@ auth_passwd_create_client_data(authorization_data_t *data,
    return tmp;
 }
 
-void binary_debug(char* msg, unsigned char *buf, int len) {
-    char *buf_copy = NULL;
-    char c;
-    int i;
-
-    buf_copy = malloc(len + 1);
-    for (i = 0; i < len; ++i) {
-	c = buf[i];
-	if (c == 0) c = '.';
-	else if (c < 32 || c > 126) c = '?';
-	buf_copy[i] = c;
-    }
-    buf_copy[len] = 0;
-    myproxy_debug("%s (%d): %s", msg, len, buf_copy);
-    free(buf_copy);
-}
-
-/*************************************************************************/
-/**                                                                     **/
-/**  code to decrypt and verify pubcookie (http://www.pubcookie.org/)   **/
-/**                                                                     **/ 
-/**        Steve Losen, UVA ITC                                         **/
-/**                                                                     **/
-/**                                                                     **/
-/*************************************************************************/
-
-/*
- * decrypt_cookie() accepts a base64 encoded input string that
- * consists of a signed cookie that may also be DES encrypted.  We
- * decrypt the input and verify the cookie data with the signature.
- *
- * inbuf  points to the base64 encoded input
- *
- * inlen  is the length of the input in bytes
- *
- * cookie points to a struct to receive the cookie data
- *
- * keybuf is a 2048 byte symmetric encryption key from which we
- *        obtain the DES key and initial vector
- *        NULL if no DES key is available (in which case cookie
- *        is asumed to be unencrypted, although still signed)
- *
- * cert   is the X509 cert for verifying the signature
- *
- * We base64 decode the input.  The last two decoded bytes are random
- * offsets into keybuf.  The first offset is the start of the DES key.
- * The second offset is the start of the initial vector.  Using the DES
- * key and initial vector, we decrypt the signature and the cookie data.
- * We verify the cookie data with the signature and the cert.  If correct,
- * then we return 0.  We return -1 on any failure.
- */
-
-int
-decrypt_cookie(const unsigned char *inbuf, int inlen,
-    struct cookie_data *cookie, const unsigned char *keybuf,
-    X509 *cert)
-{
-    unsigned char tmpbuf[2048];
-    unsigned char signature[1024];
-    int siglen;
-    BIO *bio, *b64;
-    DES_cblock deskey, ivec;
-    DES_key_schedule ks;
-    EVP_PKEY *pubkey = NULL;
-    EVP_MD_CTX ctx;
-    int offset, i;
-    int return_value = -1;
-    int is_encrypted = 0;
-
-    EVP_MD_CTX_init(&ctx);
-
-    /* 1. base64 decode the input */
-
-    if (4 * sizeof(tmpbuf) < 3 * inlen) {
-        return -1;
-    }
-    b64 = BIO_new(BIO_f_base64());
-    BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);
-    bio = BIO_new_mem_buf((void *)inbuf, inlen);
-    bio = BIO_push(b64, bio);
-
-    inlen = BIO_read(bio, tmpbuf, sizeof(tmpbuf));
-    BIO_free_all(bio);
-    inbuf = tmpbuf;
-
-    /* 2. get public key from cert and length of signature */
-
-    if ((pubkey = X509_extract_key(cert)) == 0) {
-	goto cleanup;
-    }
-
-    siglen = EVP_PKEY_size(pubkey);
-
-    /* binary_debug("inbuf", inbuf, inlen); */
-    if (siglen + sizeof(*cookie) == inlen) is_encrypted = 0;
-    else if (siglen + sizeof(*cookie) + 2 == inlen) is_encrypted = 1;
-    else {
-	myproxy_debug("decrypt_cookie: unexpected input length: %d", inlen);
-	goto cleanup;
-    }
-
-    if (siglen > sizeof(signature)) {
-	myproxy_debug("decrypt_cookie: not enough space allocated for "
-		      "signature: %d allocated; %d required.",
-		      sizeof(signature), siglen);
-	goto cleanup;
-    }
-
-    /* 3a. encrypted, so decrypt cookie data structure and signature */
-    if (is_encrypted) {
-	myproxy_debug("decrypt_cookie: cookie is encrypted.");
-	if (!keybuf) {
-	    myproxy_debug("decrypt_cookie: cookie is encrypted, "
-			  "but no DES key is provided.");
-	    goto cleanup;
-	}
-
-	/* get the DES key from keybuf */
-	offset = inbuf[inlen - 2];
-	memcpy (deskey, keybuf + offset, sizeof(deskey));
-	DES_set_odd_parity(&deskey);
-	if (DES_set_key_checked(&deskey, &ks) != 0) {
-	    goto cleanup;
-	}
-
-	/* get the DES initial vector from keybuf */
-	offset = inbuf[inlen - 1];
-	for (i = 0; i < sizeof(ivec); i++) {
-	    ivec[i] = keybuf[offset + i] ^ 0x4c;
-	}
-
-	/* decrypt signature and cookie data */
-	i = 0;
-	DES_cfb64_encrypt(inbuf, signature, siglen, &ks, &ivec, &i,
-			  DES_DECRYPT);
-
-	DES_cfb64_encrypt (inbuf + siglen, (unsigned char *)cookie,
-			   sizeof(*cookie), &ks, &ivec, &i, DES_DECRYPT);
-
-	/* binary_debug("decrypted cookie", cookie, sizeof(*cookie)); */
-	/* binary_debug("decrypted signature", signature, siglen); */
-    }
-
-    /* 3b. not encrypted, so copy cookie data structure and signature */
-    else {
-	myproxy_debug("decrypt_cookie: cookie is in plaintext");
-	memcpy(cookie, inbuf, sizeof(*cookie));
-	memcpy(signature, inbuf + sizeof(*cookie), siglen);
-
-	/* binary_debug("cookie", cookie, sizeof(*cookie)); */
-	/* binary_debug("signature", signature, siglen); */
-    }
-
-    /* 4. verify signature */
-    EVP_VerifyInit(&ctx, EVP_md5());
-    EVP_VerifyUpdate(&ctx, (unsigned char *)cookie, sizeof(*cookie));
-    if (EVP_VerifyFinal(&ctx, signature, siglen, pubkey) != 1) {
-	myproxy_debug("decrypt_cookie: invalid pubcookie signature");
-	goto cleanup;
-    }
-    myproxy_debug("decrypt_cookie: valid pubcookie signature");
-
-    /* 5. convert to host byte order */
-    cookie->pre_sess_token = ntohl(cookie->pre_sess_token);
-    cookie->create_ts      = ntohl(cookie->create_ts);
-    cookie->last_ts        = ntohl(cookie->last_ts);
-
-    return_value = 0;
-
- cleanup:
-    EVP_MD_CTX_cleanup(&ctx);
-    if (pubkey) EVP_PKEY_free(pubkey);
-
-    return return_value;
-}
-
-int auth_pubcookie_check_client (authorization_data_t *auth_data,
-				 struct myproxy_creds *creds, 
-				 char *client_name,
-				 myproxy_server_context_t* config)
-{ 
-  int return_status;
-  FILE *fp;
-  struct cookie_data cookie;
-  unsigned char keybuf[2048];
-  X509 *cert = NULL;
-  unsigned char *keyptr = NULL;
-
-  return_status = 1;
-
-  if (!config->pubcookie_cert) {
-      return 0; /* Pubcookie support not enabled. */
-  }
-
-  /* read symmetric key file, if supplied, for decrypting cookie */
-  if (config->pubcookie_key) {
-      if ((fp = fopen(config->pubcookie_key, "r")) == 0 ||
-	  fread(keybuf, 1, sizeof(keybuf), fp) != sizeof(keybuf)) {
-	  verror_put_string("ERROR opening %s", config->pubcookie_key);
-	  verror_put_errno(errno);
-	  return_status=0;
-	  if (fp)
-	      fclose(fp);
-	  /* only assign keyptr if we successfully read key file. */
-      }
-      else {
-	  keyptr = keybuf;
-      }
-  }
-  
-  /* read cert file for verifying cookie signature */
-  if(return_status==1) {
-    if ((fp = fopen(config->pubcookie_cert, "r")) == 0 ||
-        (cert = PEM_read_X509(fp, 0, 0, 0)) == 0)
-      {
-	verror_put_string("ERROR opening %s", config->pubcookie_cert);
-	verror_put_errno(errno);
-        return_status=0;
-	if (fp)
-	  fclose(fp);
-      }
-  }
-
-  /* decrypt cookie and verify  -- TO DO: make this time-dependent on the cookie, but we can't 
-     do it right now (it's NOT the create_ts -- which looks like it could be up to a week?) */
-
-  if(return_status==1) {
-    int decrypt_result;
-    int cookie_type;
-    time_t cookie_deadline, now;
-    
-    decrypt_result =   decrypt_cookie((unsigned char *)auth_data->client_data, auth_data->client_data_len, &cookie,
-                                      keyptr, cert);
-    
-    if (decrypt_result == 0) {
-      cookie_type = cookie.type;
-      cookie_deadline = cookie.create_ts + 24 * 3600;
-      
-      if (cookie_type != '1') { /* yes, fix this hard-code.. I realize... */
-        verror_prepend_string("Wrong cookie type");
-        return_status=0;
-      }
-      if(return_status==1) {
-	now = time (0);
-
-	myproxy_debug("Pubcookie presented: now is %d, cookie create_ts: %d, cookie last_ts: %d",
-		     (int) time(0), (int) cookie.create_ts, (int) cookie.last_ts);
-
-#ifdef NOT_CORRECT
-        if (cookie_deadline < now) {
-          verror_prepend_string("Cookie is older than 1 day (cookie creation timestamp: %d (%s), one day from cookie timestamp (deadline): %d (%s), now: %d (%s))", 
-				cookie.create_ts, ctime(&(cookie.create_ts)),
-				cookie_deadline, ctime(&cookie_deadline), 
-				now, ctime(&now));
-          return_status=0;
-        }
-#endif 
-      }
-    }
-    else {
-      verror_prepend_string("Could not decrypt and verify pubcookie");
-      return_status=0;
-    }
-  }
-
-  /* test #2: verify username */
-  if(return_status==1) {
-    if(strcmp((char *)cookie.user, creds->username)) {
-      verror_put_string("Pubcookie username (%s) and request username (%s) do not match", (char *)cookie.user, creds->username); 
-      return_status=0;
-    }
-  }
-
-  /* may want to verify other info at some point */
-
-  if (return_status==1) {
-      myproxy_log("Pubcookie verified username: %s", (char *)cookie.user);
-  }
-  if (cert) X509_free(cert);
-  return return_status;
-}
-/* end of Pubcookie-specific code */
-
-
 int auth_passwd_check_client(authorization_data_t *client_auth_data,
                              struct myproxy_creds *creds, char *client_name,
 			     myproxy_server_context_t* config)
@@ -380,7 +92,7 @@ int auth_passwd_check_client(authorization_data_t *client_auth_data,
    /* 1. Gather some initial information. */
    exist = myproxy_creds_exist(creds->username, creds->credname);
    if (exist < 0) {
-       return 0;
+       exist = 0; verror_clear(); /* may be in CA-only mode */
    }
    if (exist) {
        encrypted = myproxy_creds_encrypted(creds);
@@ -409,17 +121,6 @@ int auth_passwd_check_client(authorization_data_t *client_auth_data,
       }
    }
    
-   if (config && config->pubcookie_cert) {
-       config->usage.pubcookie_used = 1;
-
-       myproxy_debug("attempting pubcookie verification");
-       if (!cred_passphrase_match) {
-	   cred_passphrase_match =
-	       (auth_pubcookie_check_client(client_auth_data, creds,
-					    client_name, config) == 1) ? 1 : 0;
-       }
-   }
-
 #if defined(HAVE_LIBPAM)
 
    /* Tangent: figure out PAM configuration. */
